// Misc functions module
#include "app.h"

// Set the current directory to be the application's directory
int AppDirectory()
{
  char *szCmd=NULL,*pc=NULL; int nPathLen=0; char *szPath=NULL;
  szCmd=GetCommandLine();

  // Find the end of the "c:\directory\program.exe" bit
  if (szCmd[0]!='\"') return 1; // Error: it didn't start with a "

  for (pc=szCmd+1; *pc; pc++)
    if (*pc=='\"') break; // Find the last "

  // Find the last \ or /
  for (          ; pc>=szCmd+1; pc--)
  {
    if (*pc=='\\') break;
    if (*pc=='/') break;
  }

  nPathLen=pc-(szCmd+1);
  if (nPathLen<=0) return 1; // No path

  // Now copy the path into a new buffer
  szPath=(char *)malloc(nPathLen+1); if (szPath==NULL) return 1;
  memcpy(szPath,szCmd+1,nPathLen); szPath[nPathLen]=0;
  SetCurrentDirectory(szPath); // Finally set the current directory to be the application's directory
  free(szPath);
  return 0;
}

// This function takes a rectangle and scales it to a multiple of the
// screen resolution, so that the video hardware doesn't have to do a fractional scale
// (which looks bad in DirectDraw).
int IntegerScale(RECT *pRect,int ScrnWidth,int ScrnHeight)
{
  int xm=0,ym=0; // The multiple of ScrnWidth and ScrnHeight we can fit in
  int m=0; // The multiple of both we can fit in
  int Width=0,Height;
  int FitWidth=0,FitHeight=0;
  Width =pRect->right-pRect->left;
  Height=pRect->bottom-pRect->top;

  xm=Width/ScrnWidth; ym=Height/ScrnHeight;
  m=xm; if (m>ym) m=ym;

  if (m>=1) { FitWidth=ScrnWidth*xm; FitHeight=ScrnHeight*ym; }
  else { FitWidth=Width; FitHeight=Height; }
  
  if (Width== 640 && Height==480 && ScrnWidth==384) FitWidth=640; // 597 for exact 4/3
  if (Width== 512 && Height==384 && ScrnWidth==224) FitWidth=288; // exact 4/3
  if (Width==1024 && Height==768 && ScrnWidth==224) FitWidth=576; // exact 4/3

  pRect->left=(pRect->right+pRect->left)/2;
  pRect->left-=FitWidth>>1;
  pRect->right=pRect->left+FitWidth;

  pRect->top=(pRect->top+pRect->bottom)/2;
  pRect->top-=FitHeight>>1;
  pRect->bottom=pRect->top+FitHeight;
  return 0;
}


// Get the position of the client area of a window on the screen
int GetClientScreenRect(HWND hWnd,RECT *pRect)
{
  POINT Corner={0,0};
  GetClientRect(hWnd,pRect);
  Corner.x=0; Corner.y=0;
  ClientToScreen(hWnd,&Corner);
  pRect->left+=Corner.x; pRect->right+=Corner.x;
  pRect->top+=Corner.y; pRect->bottom+=Corner.y;
  return 0;
}

// Standard callbacks for 8/16/24/32 bit color:
static unsigned int HighCol8 (int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t=(r+g+b)/3;
  return t;
}

static unsigned int HighCol15(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<7)&0x7c00; // 0rrr rr00 0000 0000
  t|=(g<<2)&0x03e0; // 0000 00gg ggg0 0000
  t|=(b>>3)&0x001f; // 0000 0000 000b bbbb
  return t;
}

static unsigned int HighCol16(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<8)&0xf800; // rrrr r000 0000 0000
  t|=(g<<3)&0x07e0; // 0000 0ggg ggg0 0000
  t|=(b>>3)&0x001f; // 0000 0000 000b bbbb
  return t;
}

// 24-bit/32-bit
static unsigned int HighCol24(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<16)&0xff0000;
  t|=(g<<8 )&0x00ff00;
  t|=(b    )&0x0000ff;

  return t;
}

int SetBurnHighCol(int Depth)
{
  if (Depth<= 8) { BurnHighCol=HighCol8;  return 0; }
  if (Depth<=15) { BurnHighCol=HighCol15; return 0; }
  if (Depth<=16) { BurnHighCol=HighCol16; return 0; }
  BurnHighCol=HighCol24;
  return 0;
}

// Put a window in the middle of another window
int WndInMid(HWND hMid,HWND hBase)
{
  RECT MidRect={0,0,0,0};  int mw=0,mh=0;
  RECT BaseRect={0,0,0,0}; int bx=0,by=0;
  // Find the height and width of the Mid window
  GetWindowRect(hMid,&MidRect);
  mw=MidRect.right-MidRect.left;
  mh=MidRect.bottom-MidRect.top;

  // Find the center of the Base window
  GetWindowRect(hBase,&BaseRect);
  bx=BaseRect.left+BaseRect.right; bx>>=1;
  by=BaseRect.top+BaseRect.bottom; by>>=1;

  // Center the window
  SetWindowPos(hMid,NULL,bx-(mw>>1),by-(mh>>1),0,0,SWP_NOSIZE | SWP_NOZORDER);
  return 0;
}

static char szKeyName[0x54][16]=
{
  //0x00:
  "","Escape","1","2","3","4","5","6","7","8","9","0","Minus","Equals","Backspace",
  // 0x0f
  "Tab","Q","W","E","R","T","Y","U","I","O","P","[","]","Enter",
  // 0x1d
  "Left Ctrl","A","S","D","F","G","H","J","K","L","Semicolon","Apostrophe","Grave",
  // 0x2a
  "Left Shift","Backslash","Z","X","C","V","B","N","M","Comma","Period","Slash","Right Shift",
  // 0x37
  "NumPad *","Left Alt","Space","Caps Lock",
  // 0x3b
  "F1","F2","F3","F4","F5","F6","F7","F8","F9","F10",
  // 0x45
  "NumLock","ScrollLock",
  "NumPad 7","NumPad 8","NumPad 9","NumPad -",
  "NumPad 4","NumPad 5","NumPad 6","NumPad +",
  "NumPad 1","NumPad 2","NumPad 3",
  "NumPad 0","NumPad ."
};

// Print a nice name for input code c
char *InputCodeDesc(int c)
{
  static char szString[64];
  char *szName="";
  if (c>=0x4000)
  {
    int nJoy=0,nCode=0;
    nJoy=(c>>8)&0x3f;
    nCode=c&0xff;
    if (nCode>=0x10)
    {
      sprintf (szString,"Joy-%d Button %d",nJoy+1,nCode-0x0f);
      return szString;
    }
    if (nCode>=1 && nCode<=4)
    {
      static char szDir[4][8]={"Up","Down","Left","Right"};
      sprintf (szString,"Joy-%d %s",nJoy+1,szDir[nCode-1]);
      return szString;
    }
  }

  // Try and get a name for the key code
  if (c<0x54) szName=szKeyName[c]; // Get from the table of first few keys
  else if (c==0x56) szName="\\";
  else if (c==0x57) szName="F11";
  else if (c==0x58) szName="F12";
  else if (c==0x9C) szName="NumPad Enter";
  else if (c==0x9D) szName="Right Ctrl";
  else if (c==0xB5) szName="NumPad /";
  else if (c==0xB7) szName="PrintScrn";
  else if (c==0xB8) szName="Right Alt";
  else if (c==0xC5) szName="Pause";
  else if (c==0xC7) szName="Home";
  else if (c==0xC8) szName="Up Arrow";
  else if (c==0xC9) szName="PgUp";
  else if (c==0xCB) szName="Left Arrow";
  else if (c==0xCD) szName="Right Arrow";
  else if (c==0xCF) szName="End";
  else if (c==0xD0) szName="Down Arrow";
  else if (c==0xD1) szName="PgDn";
  else if (c==0xD2) szName="Insert";
  else if (c==0xD3) szName="Delete";
  else if (c==0xDB) szName="Left Win";
  else if (c==0xDC) szName="Right Win";
  else if (c==0xDD) szName="AppMenu";

  if (szName[0]) sprintf (szString,"%s",szName);
  else sprintf (szString,"Key 0x%.2x",c);
  return szString;
}
